/*
 *  client/functions-read.c
 *		The functions to actually execute the for the
 *		client.  Commands to read texts.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */



#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#include <config.h>
#include "config.h"

#include <kom-errno.h>
#include <kom-types.h>
#include <services.h>
#include <zmalloc.h>
#include <s-string.h>
#include <s-collat-tabs.h>

#include "xoutput.h"
#include "copy.h"
#include "read-line.h"
#include "commands.h"
#include "parser.h"
#include "texts.h"
#include "edit.h"
#include "misc-parser.h"
#include "error.h"
#include "cache.h"
#include "quit.h"

#include "internal.h"
#include "cmds.h"


/* This is to emphasize that an object is exported. */
#define Export

/* We are keeping track of whats going on here. */
Export Internal_state_t	reading_state;


/*
 *  The parameter 'ARGUMENT' in all the external functions, is a
 *  String containing the rest of the line, following the command.
 *  E g if the user gave the command "Skapa m|te Inl{gg }t mig",
 *  then the String "Inl{gg }t mig" is passed as the parameter
 *  'ARGUMENT' to the function responsible for the command 'skapa
 *  m|te'.  Leading and trailing spaces are skipped.  EMPTY_STRING
 *  is passed if no arguments to the command was given.
 */


/*  Find and display text(s) according to strange search patterns.  */
Export  Success
cmd_find_and_view_text (String   argument)
{
    Text_no		  wanted_text;
    const String	  the_commented	= s_fcrea_str(txts.the_commented);
    String_size		  error_check;	/* First uncoverted char str->num */
    Success		  retval;


#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    if (s_empty(argument))
    {
	xprintf("%s?\n= ", txts.What_texts_to_view);
	read_line (&argument);		/* BUG! Check retval! */
	newline();
    }
    argument = s_strip_trailing (argument, command_separators);

    /* A number, and nothing else? */
    wanted_text = (Text_no) s_strtol (argument, &error_check,
				      DEFAULT_NUMBER_BASE);
    if (error_check < s_strlen (argument))
    {
	/* No, it wasn't a number.  Then try "(the) commented" */
	/* ??? Check this out */
	if (! s_usr_strhead (argument, the_commented, DEFAULT_COLLAT_TAB))
	{
	    xprintf ("%s: ", txts.Unknown_argument);
	    s_puts(argument);
	    newline();
	    RETURN (FAILURE);
	}
	cmd_not_yet_implemented (argument);
	RETURN (FAILURE);
    }
    else
    {
	/* OK, it was a number */
	retval = show_text_no (wanted_text);
	RETURN (retval);
    }

    RETURN (OK);
#undef CLEAN_UP
}




/*  Read the next text in order  */
Export  Success
cmd_read_next_text (String   argument)

{
#define FUNCTION "read_next_text()"

    Success		  exit_value;
    Local_text_no	  text_to_read;
    Text_no		  global_no_to_read;
    Kom_err		  error;


#define CLEAN_UP()	do {	} while (0)


    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (find_next_unread_text (&text_to_read) == FAILURE)
    {
	switch (kom_errno)
	{
	case  KOM_UNDEF_CONF:
	case  KOM_PERM:
	    xprintf ("%s.\n", txts.Not_allowed_to_read_anymore);
	    break;

	default:
	    fatal2 (CLIENT_SHOULDNT_HAPPEN,
		    "find_next_unread_text(%d) returns bogus",
		    text_to_read);
	    break;
	}

	RETURN (FAILURE);
    }

    global_no_to_read = map_local_to_global (ccn, text_to_read, &error);
    /* BUG: Doesn't check for error */
    exit_value = show_text_no (global_no_to_read);

    /* The text will be marked as read when the next command is given. */
    /* But the counting is done here. */
    decrease_unread_texts_left();

    reading_state.last_normally_read = global_no_to_read;
    reading_state.local_last_normally_read = text_to_read;

    RETURN (exit_value);

#undef CLEAN_UP
#undef FUNCTION
}


/*  Read the commented text  */
extern Success
cmd_read_commented_text (String  argument)
{
#define FUNCTION "read_commented_text()"

    Text_no		  text_to_search_from;
    Text_no		  text_to_read = 0;
    Text_stat		  text_stat = EMPTY_TEXT_STAT;

#define CLEAN_UP()	do { s_clear (&argument); } while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    text_to_search_from = reading_state.last_viewed;

    if ( text_to_search_from
	 && kom_get_text_stat (text_to_search_from,
			       &text_stat ) != FAILURE)
    {
	Misc_info     * misc;
	int		r;

	newline();
	for (misc = text_stat.misc_items , r = 0;
	     r < text_stat.no_of_misc && !text_to_read;
	     r++, misc++)
	{
	    if (misc->type == comm_to)
	    {
		text_to_read = misc->datum.comment_to;
	    }
	}

	if (text_to_read)
	    RETURN (show_text_no (text_to_read));
	else {
	    xprintf("%s.\n", txts.Text_doesnt_exist);
	    RETURN (OK);
	}
    }
    
    RETURN (FAILURE);
#undef CLEAN_UP
#undef FUNCTION
}




/*  Goto the next conference with unread news. */
extern Success
cmd_goto_next_conf (String   argument)
{
#define FUNCTION "cmd_goto_next_conf()"

    Kom_err		  error;

    Conf_no_list	  list = EMPTY_CONF_NO_LIST;
    Membership 		  ship = EMPTY_MEMBERSHIP;
    Membership_list	  ship_list = EMPTY_MEMBERSHIP_LIST;
    int 		  pos;
    
#define CLEAN_UP()	do {	release_conf_no_list (&list); \
				release_membership_list (&ship_list); \
			} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (kom_get_unread_confs (cpn, &list) == FAILURE)
    {
	switch (kom_errno)
	{
	case  KOM_OUT_OF_MEMORY:
	    fatal1 (CLIENT_OUT_OF_MEMORY, "get_conf_no_list() failed");
	    break;

	case  KOM_NO_CONNECT:
	    fatal1 (CLIENT_SERVER_ERROR, "get_conf_no_list() failed");
	    break;

	default:
	    fatal1 (CLIENT_UNEXPECTED_SERVER_ERROR,
		    "get_conf_no_list() failed");
	    break;
	}
    }

    for (pos = first_membership(cpn, &ship);
	 pos != -1;
	 pos = next_membership(&ship))
    {
	if ( membership_in_conf_no_list(ship, list) )
	{
	    int unread;

	    unread = conf_last(ship.conf_no, &error)
		- (ship.last_text_read + ship.no_of_read);
	    if (unread > 0) {
		xprintf("%S - %d %s.\n", conf_name(ship.conf_no, &error),
			unread, 
			(unread == 1 ? txts.unread_singular
			             : txts.unread_plural));
		goto_conference(ship.conf_no);
		RETURN (OK);
	    }
	}
    }
    
    RETURN (FAILURE);

#undef CLEAN_UP
#undef FUNCTION
}



/* ==================================================================== */
/*			Internal functions				*/


/*
 *  Fetch and show the text WANTED if the server is willing to let
 *  the person see it.  Currently only one format for showing the
 *  texts - no options available.
 */
Export  Success
show_text_no (Text_no	wanted)

{
/*    static char		* this_function	= "show_text_no()";*/

    Text_stat	*volatile stat		= NULL;
    String		  text		= EMPTY_STRING;
    Kom_err		  error;


#define CLEAN_UP()	do {				\
	if (stat != NULL)				\
	    zfree (stat->misc_items);			\
	zfree (stat);					\
	s_clear (&text);					} while (0)

    TOPLOOP_SETJMP();

    if (  (stat = text_stat (wanted, &error)) == NULL
	|| s_empty (text = get_text (wanted, 0, END_OF_STRING, &error)))
    {
	xprintf ("%s.\n", txts.Text_doesnt_exist);
	RETURN (FAILURE);
    }

    print_text_header (wanted, stat);
    print_misc_list (stat->misc_items, stat->no_of_misc,
		     PRINT_RECPT | PRINT_CC_RECPT);
    print_misc_list (stat->misc_items, stat->no_of_misc,
		     PRINT_COMM_TO | PRINT_FOOTN_TO);
    print_subject_line (txts.Subject, text);
    xprintf ("------------------------------------------------------------\n");
    print_rest_of_text (text);
    xprintf ("(%lu)------------------------------\n", wanted);
    print_misc_list (stat->misc_items, stat->no_of_misc,
		     PRINT_COMM_IN | PRINT_FOOTN_IN);
    reading_state.last_viewed = wanted;

    RETURN (OK);

#undef CLEAN_UP
}



Export  void
print_text_header (Text_no		  number,
		   const Text_stat	* stat   )
{
    xprintf ("(%lu) ", number);
    print_time (&stat->creation_time);
    xprintf (" /%d %s/ ", stat->no_of_lines, txts.lines);
    print_conf_name ((Conf_no) stat->author);
    newline();
}



Export  void
print_misc_list (const Misc_info	* misc_list,
		 u_short		  no_of_misc,
		 u_short		  what_to_print)

{
#define FUNCTION "print_misc_list()"

    const Misc_info	* stop_ptr;	/* Address of element beyond last */
    Misc_info_group	  misc_data;
    Text_stat		* stat2;	/* For info about other texts */
    Kom_err		  error;


    stop_ptr = misc_list + no_of_misc;
    while (  (misc_data = parse_next_misc (&misc_list, stop_ptr)).type
	   != m_end_of_list)
    {
	switch (misc_data.type)
	{
	case  m_recpt:
	    if (! (what_to_print & PRINT_RECPT))
		break;
	    /* Else fall through */

	case  m_cc_recpt:
	    if (! (what_to_print & (PRINT_RECPT | PRINT_CC_RECPT)))
		break;

	    if (misc_data.type == m_recpt) {
		if (is_sent_by (misc_data))
		    xprintf ("%s: ", txts.Carbon_copy);
		else
		    xprintf ("%s: ", txts.Recipient);
		print_conf_name (misc_data.recipient);
	    }
	    else {
		xprintf ("%s: ", txts.Extra_copy);
		print_conf_name (misc_data.cc_recipient);
	    }
	    if (misc_data.local_no) /* If we have a local_no */
		xprintf (" <%lu>", misc_data.local_no);

	    if (is_received (misc_data))
	    {
		xprintf (" -- %s: ", txts.Received);
		print_time (&misc_data.received_at);
	    }

	    newline();

	    if (is_sent_by (misc_data))
	    {
		xprintf (" %s: ", txts.Sender);
		print_conf_name ((Conf_no) misc_data.sender);
	    }
	    if (is_sent (misc_data))
	    {
		xprintf (" -- %s: ", txts.Sent_at);
		print_time (&misc_data.sent_at);
		newline();
	    }

	    break;


	case  m_comm_to:
	    if (! (what_to_print & PRINT_COMM_TO))
		break;

	    xprintf ("%s %lu", txts.Comment_to_text,
		     misc_data.comment_to);
	    if ((stat2 = text_stat (misc_data.comment_to, &error)) != NULL)
	    {
		xprintf (" %s ", txts.by);
		print_conf_name ((Conf_no) stat2->author);
		zfree (stat2->misc_items);
		zfree (stat2);
	    }
	    newline();
	    break;


	case  m_footn_to:
	    if (! (what_to_print & PRINT_FOOTN_TO))
		break;

	    xprintf ("%s %lu", txts.Footnote_to_text,
		     misc_data.footnote_to);
	    if ((stat2 = text_stat (misc_data.footnote_to, &error)) != NULL)
	    {
		xprintf (" %s ", txts.by);
		print_conf_name ((Conf_no) stat2->author);
		zfree (stat2->misc_items);
		zfree (stat2);
	    }
	    newline();
	    break;


	case  m_comm_in:
	    if (! (what_to_print & PRINT_COMM_IN))
		break;

	    xprintf ("(%s %lu", txts.Comment_in_text,
		     misc_data.commented_in);
	    if ((stat2 = text_stat (misc_data.commented_in, &error)) != NULL)
	    {
		xprintf (" %s ", txts.by);
		print_conf_name ((Conf_no) stat2->author);
		zfree (stat2->misc_items);
		zfree (stat2);
	    }
	    xputchar(')');
	    newline();
	    break;


	case  m_footn_in:
	    if (! (what_to_print & PRINT_FOOTN_IN))
		break;

	    xprintf ("(%s %lu", txts.Footnote_in_text,
		     misc_data.footnoted_in);
	    if ((stat2 = text_stat (misc_data.footnoted_in, &error)) != NULL)
	    {
		xprintf (" %s ", txts.by);
		print_conf_name ((Conf_no) stat2->author);
		zfree (stat2->misc_items);
		zfree (stat2);
	    }
	    xputchar(')');
	    newline();
	    break;


	case  m_error:
	    server_bug ("Bad misc-items", FUNCTION,
			(bug_info) misc_list);
	    break;


	case  m_end_of_list:
	    abort();	/* This case is taken care of by the while */
	}
    }	/* while (! end-of-misc-list) */
#undef FUNCTION
}



Export  void
print_subject_line (char		* subject_text,
		    const String	text		)
{
    String_size		  end_of_line_pos;


    xprintf ("%s:  ", subject_text);
    end_of_line_pos = s_strchr (text, '\n', 0);
    s_xputs (s_fsubstr (text, 0, (end_of_line_pos == -1
				  ? END_OF_STRING
				  : end_of_line_pos)));
}



Export  void
print_rest_of_text (const String	text)
{
    String_size		  end_of_line_pos;


    end_of_line_pos = s_strchr (text, '\n', 0);
    s_xputs (s_fsubstr (text, end_of_line_pos + 1, END_OF_STRING));
    if (text.string[s_strlen (text) - 1] != '\n')
	newline();
}



Export  void
print_time (const struct tm	* time)
{
    xprintf ("%02d-%02d-%02d %02d.%02d",
	     time->tm_year, time->tm_mon+1,
	     time->tm_mday, time->tm_hour, time->tm_min);
}



/*
 *  Find the next unread text in the current conference and
 *  put the result in NEXT_TO_READ.  Gives the result 0 if
 *  there are no more unread.
 *  Returns OK if everything went fine, and FAILURE if something
 *  (non-fatal) went wrong, e g couldn't get the status of the
 *  current conference.  The global variable KOM_ERRNO is set
 *  to the error.
 */
/* BUG: does not check if a text already has been read,
   via a "]terse"-command, or in another conference	*/
Export  Success
find_next_unread_text (Local_text_no	* next_to_read)

{
#define FUNCTION "find_next_unread_text()"
    Kom_err		error;
    int 		to_read;
    Text_no 		text;
    Membership		ship;

#define CLEAN_UP()	do {	} while (0)

    TOPLOOP_SETJMP();

    if (locate_membership(ccn, cpn, &ship) == -1)
    {				/* There is a problem with this one... */
	fatal2 (CLIENT_ERROR_TOO_boring,
		"membership of current conf (%s) not found. Conf deleted, you are thrown out...",
		ccn);
    }

    to_read = ship.last_text_read + 1;
    while ((text = map_local_to_global (ccn,to_read,&error)) == 0
 	   && kom_errno == KOM_NO_ERROR) {
	mark_as_read( ccn, 1, &to_read );
  	to_read++;
	decrease_unread_texts_left();
    }
    * next_to_read = to_read;

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}


#if 0
/*
 *  Find the next comment to ROOT_TEXT, after THIS_COMMENT, or the
 *  first comment if THIS_COMMENT is 0.  Returns 0 if there are no
 *  more comments to ROOT_TEXT.
 */
Export  Text_no
next_comment (Text_no	  root_text,
	      Text_no	  this_comment)

{
    Text_stat		* stat		= NULL;
    Kom_err		  error;
    Misc_info_group	  misc;
    Misc_info		* info_list;
    Misc_info		* end_ptr;

#define CLEAN_UP()

    TOPLOOP_SETJMP();

    stat = text_stat (root_text, &error);
    if (stat == NULL)
    {
	RETURN (0);
    }

    info_list = stat->misc_items;
    end_ptr = info_list + stat->no_of_misc;

    if (this_comment != 0)
    {
	do
	{
	    misc = parse_next_misc (&info_list, end_ptr);
	} while (    misc.type != m_end_of_list
		 &&  (  misc.type != m_comm_to
		      ||  misc.comment_to != this_comment));
	if (misc.type == m_end_of_list)
	{
	    RETURN (0);
	}
    }

    do
    {
	misc = parse_next_misc (&info_list, end_ptr);
    } while (    misc.type != m_end_of_list
	     &&  misc.type != m_comm_to);
    if (misc.type == m_end_of_list)
    {
	RETURN (0);
    }

    RETURN (misc.comment_to);

#undef CLEAN_UP
}



/*
 *  Find the next text in conference CONF, following the text TEXT, or
 *  0 if there are no more texts in the conference.
 */
Export  Text_no
next_in_conf (Conf_no	  conf,
	      Text_no	  text)

{
    Local_text_no	  local;
    Text_stat		* stat		= NULL;
    Kom_err		  error;
    Misc_info_group	  misc;
    Misc_info		* info_list;
    Misc_info		* end_ptr;
    Text_no		  next_text;

#define CLEAN_UP()	do { zfree (release_text_stat (stat)); } while (0)

    TOPLOOP_SETJMP();

    stat = text_stat (text, &error);
    if (stat == NULL) 
    {
	RETURN (0);
    }

    info_list = stat->misc_items;
    end_ptr = info_list + stat->no_of_misc;
    do
    {
	misc = parse_next_misc (&info_list, end_ptr);
    } while (    misc.type != m_end_of_list
	     &&  misc.type != m_recpt
	     &&  misc.type != m_cc_recpt);

    if (misc.type == m_end_of_list)
    {
	RETURN (0);
    }

    next_text = 0;
    local = misc.local_no;
    while (next_text == 0) 
    {
	next_text = map_local_to_global (conf, ++local, &error);
	if (next_text == 0  &&  error == KOM_NO_SUCH_LOCAL_TEXT)
	{
	    RETURN (0);
	}
    }

    RETURN (next_text);

#undef CLEAN_UP
}   /* END: next_in_conf() */



/*
 *  Check if a text has been read during this session, or has been
 *  read in this conference earlier.
 */
Export  Bool
has_been_read (Text_no		  text,
	       Conf_no		  conf)

{
    long			  i;
    struct read_text_list	* list;

#define CLEAN_UP()	do {	} while (0)

    TOPLOOP_SETJMP();

    list = &read_texts[text % READ_TEXT_HASH];

    i = list->no_of_texts - 1;
    while (i >= 0  &&  list->text_list[i].text != text)
	i--;

    if (i >= 0)
    {
	RETURN (TRUE);
    }
}
#endif



/*
 *  Report the number of unread texts the person WHO has in
 *  conference WHERE.
 */
Export  long
unread_in_conf (Pers_no	    who,
		Conf_no	    where)

{
    Membership		  membship	= EMPTY_MEMBERSHIP;
    long		  no_of_unread;
    Kom_err		  error;

#define CLEAN_UP()	do { } while (0)

    TOPLOOP_SETJMP();

    if (locate_membership (where, who, &membship) < 0)
    {
	RETURN (-1);
    }

    no_of_unread = (conf_last(where, &error)
		    - membship.last_text_read - membship.no_of_read);

    RETURN (no_of_unread);

#undef CLEAN_UP
}   /* END: unread_in_conf() */




#define NOTYETREAD -1

/* How many articles are there left to read in this conference? */
static int more_to_read = NOTYETREAD;
static Conf_no more_to_read_for_conf; 

/* Returns the value of more_confs i.e the number of unread confs left.
 */
extern int
unread_confs_left()
{
#define FUNCTION "unread_confs_left()"

    Conf_no_list	  list = EMPTY_CONF_NO_LIST;
    int			  count_of_uconfs;

    if (kom_get_unread_confs (cpn, &list) == FAILURE) {
	switch (kom_errno) {
	case  KOM_OUT_OF_MEMORY:
	    fatal1 (CLIENT_OUT_OF_MEMORY, "get_conf_no_list() failed");
	    break;

	case  KOM_NO_CONNECT:
	    fatal1 (CLIENT_SERVER_ERROR, "get_conf_no_list() failed");
	    break;

	default:
	    fatal1 (CLIENT_UNEXPECTED_SERVER_ERROR, 
		    "get_conf_no_list() failed");
	    break;
	}
    }

    more_to_read = NOTYETREAD;
    more_to_read_for_conf = 0;
    count_of_uconfs = list.no_of_confs;
    release_conf_no_list(&list); 
    return count_of_uconfs;
#undef FUNCTION
}



/* Returns the amount of texts left in this conf to be read.
 * If we havnt read the amount of unread then get them.
 * Returns 0 if not in any conf or there are 0 unread left.
 */
extern int
unread_texts_left()
{
    if (ccn == 0) {
	more_to_read = NOTYETREAD;
	more_to_read_for_conf = 0;
	return 0;
    }
    if (more_to_read == NOTYETREAD
	|| ccn != more_to_read_for_conf
	|| more_to_read == 0) {
	more_to_read = unread_in_conf(cpn,ccn);
	more_to_read_for_conf = ccn;
    }
    return more_to_read;
}


/* Decreases the amount of unread texts.
 * This is to get a quicker count.
 */
extern int
decrease_unread_texts_left()
{
    return --more_to_read;
}


