/*
 * $Id: input.c,v 0.4 1991/11/11 00:30:42 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 *  services.c  --  All the services the LysKOM server makes available
 *		    for clients.
 *
 *  Created by ceder 1990-07-17
 */

static char *rcsid = "$Id: input.c,v 0.4 1991/11/11 00:30:42 ceder Exp $";

#include <kom-types.h>
#include <config.h>
#include <services.h>
#include <kom-errno.h>
#include "isc-parse.h"
#include "isc-client.h"
#include "isc-async.h"
#include "input.h"


extern Success
input_header(void)
{
    int c;
    
    rewind(KS->fp);
    
    skipwhite(KS->fp);

    while ( (c=getc(KS->fp)) == ':' )
    {
	if ( async(KS->fp) != OK )
	    return FAILURE;
	
	skipwhite(KS->fp);
    }
    
    switch(c)
    {
    case '=':
	(void)parse_long(KS->fp);	/* Don't care abot ref_no */
	kom_errno = KOM_NO_ERROR;
	return OK;
    case '%':
	if ( (c=getc(KS->fp)) == '%' )
	{
	    /* The server is sending '%% LysKOM protocol error.' */
	    /* BUG: Since the line is not discarded it is harder
	     *      than it ought to be to recover from this error. */
	    kom_errno = KOM_CLIENT_IS_CRAZY;
	    return FAILURE;
	}
	ungetc(c, KS->fp);
	(void)parse_long(KS->fp);
	kom_errno = parse_long(KS->fp);
	err_stat = parse_long(KS->fp);
	return FAILURE;
	
    default:
	kom_errno = KOM_SERVER_IS_CRAZY;
	return FAILURE;
    }
}

	

extern  Pers_no
input_create_person (const String  name,
		     const String  passwd )
{
    if ( input_header() != OK )
	return 0;

    return parse_long(KS->fp);
}


extern  Success
input_get_person_stat (Person        * result )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_person(KS->fp, result);
}

extern  Success
input_get_created_texts(Text_list   * result)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_text_list(KS->fp, result);
}


extern  Success
input_get_membership(Membership_list  * memberships)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_membership_list(KS->fp, memberships);
}


extern  Success
input_query_read_texts ( Membership * result   )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_membership(KS->fp, result );
}


extern  Conf_no
input_create_conf (const String  name,
		   Conf_type	   type )
{
    if ( input_header() != OK )
	return 0;

    return parse_long(KS->fp);
}




extern  Success
input_lookup_name (Conf_list     *result)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_conf_list(KS->fp, result );
}



extern  Success
input_get_conf_stat (Conference     * result )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_conference(KS->fp, result );
}

extern  Success
input_get_members (Member_list	* members)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_member_list(KS->fp, members);
}


extern  Success
input_get_marks (Mark_list *result )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_mark_list(KS->fp, result);
}



extern  Success
input_get_text (String  *result)
{
    if ( input_header() != OK )
	return FAILURE;
    
    return parse_string(KS->fp, result);
}



extern  Success
input_get_text_stat (Text_stat *result)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_text_stat(KS->fp, result);
}



extern  Text_no
input_create_text (void)
{
    if ( input_header() != OK )
	return 0;

    return parse_long(KS->fp);
}


extern  Text_no
input_create_anonymous_text (void)
{
    if ( input_header() != OK )
	return 0;

    return parse_long(KS->fp);
}

extern  Success
input_get_map (Text_list  * result)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_text_list(KS->fp, result);
}



extern  Success
input_get_time ( struct tm *time )
{
    if ( input_header() != OK || parse_time(KS->fp, time) != OK )
	return FAILURE;

    return OK;
}


extern  Success
input_who_is_on (Who_info_list *result)
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_who_info_list(KS->fp, result);
}


extern  Success
input_get_info (Info *result )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_info(KS->fp, result);
}


extern  Success
input_get_unread_confs (Conf_no_list *result )
{
    if ( input_header() != OK )
	return FAILURE;

    return parse_conf_no_list (KS->fp, result);
}


