/*
 * $Id: isc-client.c,v 0.6 1993/01/25 21:24:28 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** client.c                        definitions of ISC client support routines
**
** history:
**    900509 pen       initial coding
*/

static char *rcsid = "$Id: isc-client.c,v 0.6 1993/01/25 21:24:28 linus Exp $";

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>  
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>

#include <sys/socket.h>

#include <kom-types.h>

#include "isc-malloc.h"
#include "isc-client.h"

/*
 * Open a connection to a server. host == NULL means this machine.
 */
ISC *
isc_connect(const char *host,
	    int port)
{
    ISC *isc_si;
    char buf[100];
    struct sockaddr_in server;
    struct hostent *hp;



    /* Allocate storage for the ISC structure */
    isc_si = (ISC *) isc_malloc(sizeof(ISC));


    /* Local connection? */
    if (host == NULL)
    {
	gethostname(buf, sizeof(buf));
	host = buf;
    }

    if ((hp = gethostbyname(host)) != NULL)
    {
	bzero((char *) &server, sizeof(server));
#ifdef NETDB_DIFF
	bcopy(hp->h_addr_list, (char *) &server.sin_addr, hp->h_length);
#else
	bcopy(hp->h_addr, (char *) &server.sin_addr, hp->h_length);
#endif
	server.sin_family = hp->h_addrtype;
    }
    else
    {
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(host);
	if (server.sin_addr.s_addr == -1)
	    return NULL;
    }

    server.sin_port = (int) htons(port);
    if ((isc_si->fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	return NULL;

    setsockopt(isc_si->fd, SOL_SOCKET, ~SO_LINGER, 0, 0); 
    setsockopt(isc_si->fd, SOL_SOCKET, SO_REUSEADDR, 0, 0);
    setsockopt(isc_si->fd, SOL_SOCKET, SO_KEEPALIVE, 0, 0);

    if (connect(isc_si->fd, (struct sockaddr *)&server, sizeof(server)) < 0
	|| (isc_si->fp = fdopen(isc_si->fd, "r+" )) == NULL )
    {
	close(isc_si->fd);
	return NULL;
    }
    setbuf(isc_si->fp, NULL);
    
    return isc_si;
}


/*
 * Shut down the connection
 */
Success
isc_disconnect(ISC *isc_si)
{
    fclose(isc_si->fp);		/* *** check retval */
    isc_free(isc_si);
    return OK;
}



/*
 * Wait for a message from the server. I opt_readset != NULL also wait for
 * messages on those fds.
 *
 * Return value:
 *	< 0	error
 *	> 0	there is a message from the server (and maybe also
 *		in opt_readset)
 *	= 0	there is a message in the opt_readset.
 */
int
isc_wait(ISC *isc_si,
	 int timeout,
	 fd_set *opt_readset)
{
    fd_set read_set;
    struct timeval wait;
    int nfds;


    /* Setup timeout structure */
    wait.tv_sec  = timeout / 1000;
    wait.tv_usec = (timeout * 1000) % 1000000;

    /* Setup file descriptor set */
    if ( opt_readset == NULL )
    {
	FD_ZERO(&read_set);
	opt_readset = &read_set;
    }
    FD_SET(isc_si->fd, opt_readset);

  
    nfds = select(FD_SETSIZE,
		  opt_readset,
		  (fd_set *) NULL,
		  (fd_set *) NULL,
		  &wait);


    return (nfds < 0 ? nfds : FD_ISSET(isc_si->fd, opt_readset));
}
 
    
