/*
 * $Id: isc-parse.c,v 0.4 1991/12/17 22:18:38 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 * isc-parse.c  -  receive objects from an isc connection.
 *
 * Written by ceder 1990-07-13
 */

static char *rcsid = "$Id: isc-parse.c,v 0.4 1991/12/17 22:18:38 ceder Exp $";

/*
 * KNOWN BUGS: Does not read in the array when returning KOM_SERVER_NO_MEM.
 *	       Thus the client will be out of sync after that error.
 * SOLUTION:   Use skip_token from parse-async.c (not yet written).
 */
/* Error checking is poor. */

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/file.h>
#include <stdlib.h>
#include <time.h>

#include <kom-types.h>
#include <kom-errno.h>
#include <services.h>

#include "isc-malloc.h"
#include "isc-parse.h"
#include "isc-async.h"


#define REALLOC(ptr, size)	(((ptr) == NULL) ?		\
					  isc_malloc (size)		\
					: isc_realloc ((ptr), (size)))


/* +++ Should have some error checking here. */

u_long
parse_long(FILE *fp)
{
    u_long res=0;

    fscanf(fp, " %lu", &res);
    return res;
}


void
skipwhite(FILE *fp)
{
    int c;

    while ( isspace(c = getc(fp)) )
	;

    ungetc(c, fp);
}


Success
parse_person (FILE *fp,
	      Person *person)
{
    if ( parse_string(fp, &person->username) != OK
	|| parse_priv_bits (fp, &person->privileges) != OK
	|| parse_personal_flags (fp, &person->flags) != OK )
    {
	return FAILURE;
    }

    parse_time(fp, &person -> last_login);

    person -> user_area		= parse_long(fp);
    person -> total_time_present = parse_long(fp);
    person -> sessions		= parse_long(fp);
    person -> created_lines	= parse_long(fp);
    person -> created_bytes	= parse_long(fp);
    person -> read_texts	= parse_long(fp);
    person -> no_of_text_fetches = parse_long(fp);
    person -> created_persons	= parse_long(fp);
    person -> created_confs	= parse_long(fp);
    person -> first_created_text = parse_long (fp);
    person -> no_of_created_texts = parse_long (fp);
    person -> no_of_marks	= parse_long (fp);
    person -> no_of_confs	= parse_long (fp);

    return OK;
}

Success
parse_membership(FILE *fp,
		 Membership *mship)
{
    int i;

    parse_time(fp, &mship->last_time_read);
    
    mship->conf_no = parse_long(fp);
    mship->priority = parse_long(fp);
    mship->last_text_read = parse_long(fp);
    mship->no_of_read = parse_long(fp);

    if ( mship->no_of_read > 0)
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    mship->read_texts = REALLOC(mship->read_texts,
					(mship->no_of_read
					 * sizeof(Local_text_no)));
	    
	    if ( mship->read_texts == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }

	    for ( i = 0; i < mship->no_of_read; i++)
		mship->read_texts[ i ] = parse_long(fp);

	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    break;
	    
	case '*':
	    isc_free(mship->read_texts);
	    mship->read_texts = NULL;
	    break;
	    
	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}

	isc_free(mship->read_texts);
	mship->read_texts = NULL;
    }

    return OK;
}

	
extern Success
parse_membership_list(FILE *fp,
		      Membership_list *result)
{
    int i;

    /* First free all the read_texts. */

    if ( result->confs != NULL )
    {
	for ( i = 0; i < result->no_of_confs; i++)
	    isc_free(result->confs[ i ].read_texts);
    }
    
    result->no_of_confs = parse_long(fp);

    if ( result->no_of_confs > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->confs = REALLOC(result->confs,
				    (result->no_of_confs
				     * sizeof(Membership)));
	    if ( result->confs == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }

	    for ( i = 0; i < result->no_of_confs; i++)
	    {
		result->confs[ i ] = EMPTY_MEMBERSHIP;
		if ( parse_membership(fp, &result->confs[i]) != OK )
		    return FAILURE;
	    }
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;
	case '*':
	    if ( result->confs != NULL )
	    {
		isc_free(result->confs);
		result->confs = NULL;
	    }
	    break;
	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->confs != NULL )
	{
	    isc_free(result->confs);
	    result->confs = NULL;
	}
    }
    return OK;
}


extern Success
parse_conf_list(FILE *fp,
		Conf_list *result)
{
    int i;

    result->no_of_confs = parse_long(fp);
    if ( result->no_of_confs > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->confs = REALLOC(result->confs,
				    (result->no_of_confs
				     * sizeof(Micro_conf)));
	    if ( result->confs == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_confs; i++)
		result->confs[ i ].conf_no = parse_long(fp);
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;
	case '*':
	    if ( result->confs != NULL )
	    {
		isc_free(result->confs);
		result->confs = NULL;
	    }
	    break;
	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->confs != NULL )
	{
	    isc_free(result->confs);
	    result->confs = NULL;
	}
    }

    if ( result->no_of_confs > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    if ( result->confs == NULL )
	    {			/* Always both conf_no and conf_type,
				   nicht wahr? */
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_confs; i++)
		if ( parse_conf_type(fp, &result->confs[i].type) != OK )
		{
		    return FAILURE;
		}
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;
	case '*':
	    if ( result->confs != NULL )
	    {
		isc_free(result->confs);
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    break;
	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->confs != NULL )
	{
	    isc_free(result->confs);
	    result->confs = NULL;
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    return OK;
}

extern Success
parse_conference(FILE *fp,
		 Conference *result)
{
    if ( parse_string(fp, &result->name) != OK
	|| parse_conf_type(fp, &result->type) != OK)
    {
	return FAILURE;
    }
    
    parse_time (fp, &result -> creation_time);
    parse_time (fp, &result -> last_written );

    result -> creator		= parse_long (fp);
    result -> presentation	= parse_long (fp);
    result -> supervisor	= parse_long (fp);
    result -> permitted_submitters = parse_long (fp);
    result -> super_conf	= parse_long (fp);
    result -> msg_of_day	= parse_long (fp);
    result -> nice		= parse_long (fp);
    result -> no_of_members	= parse_long (fp);
    result -> first_local_no	= parse_long (fp);
    result -> no_of_texts	= parse_long (fp);

    return OK;
}

extern Success
parse_mark_list(FILE *fp,
		Mark_list *result)
{
    int i;

    result->no_of_marks = parse_long(fp);

    if ( result->no_of_marks > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->marks = REALLOC(result->marks,
				    (result->no_of_marks
				     * sizeof(Mark)));

	    if ( result->marks == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_marks; i++)
		if ( parse_mark(fp, &result->marks[ i ] ) != OK )
		{
		    return FAILURE;
		}
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->marks != NULL )
	    {
		isc_free(result->marks);
		result->marks = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->marks != NULL )
	{
	    isc_free(result->marks);
	    result->marks = NULL;
	}
    }
    return OK;
}


extern Success
parse_text_stat(FILE *fp,
		Text_stat *result)
{
    int i;

    parse_time(fp, &result->creation_time);

    result->author = parse_long(fp);
    result->no_of_lines = parse_long(fp);
    result->no_of_chars = parse_long(fp);
    result->no_of_marks = parse_long(fp);
    result->no_of_misc = parse_long(fp);

    if ( result->no_of_misc > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->misc_items = REALLOC(result->misc_items,
				    (result->no_of_misc
				     * sizeof(Misc_info)));

	    if ( result->misc_items == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_misc; i++)
		if ( parse_misc_info(fp, &result->misc_items[ i ]) != OK )
		{
		    return FAILURE;
		}
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->misc_items != NULL )
	    {
		isc_free(result->misc_items);
		result->misc_items = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->misc_items != NULL )
	{
	    isc_free(result->misc_items);
	    result->misc_items = NULL;
	}
    }
    return OK;
}	


extern Success
parse_text_list(FILE *fp,
		Text_list *result)
{
    int i;

    result->first_local_no = parse_long(fp);
    result->no_of_texts = parse_long(fp);

    if ( result->no_of_texts > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->texts = REALLOC(result->texts,
				    (result->no_of_texts
				     * sizeof(Text_no)));

	    if ( result->texts == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_texts; i++)
		result->texts[ i ] = parse_long(fp);
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->texts != NULL )
	    {
		isc_free(result->texts);
		result->texts = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->texts != NULL )
	{
	    isc_free(result->texts);
	    result->texts = NULL;
	}
    }
    return OK;
}
    

extern Success
parse_info(FILE *fp,
	   Info *result)
{
    result->version = parse_long(fp);
    result->conf_pres_conf = parse_long(fp);
    result->pers_pres_conf = parse_long(fp);
    result->motd_conf = parse_long(fp);
    result->kom_news_conf = parse_long(fp);
    result->motd_of_lyskom = parse_long(fp);
    return OK;
}


extern Success
parse_string(FILE *fp,
	     String *result)
{
    String_size i;
    int c;
    
    result->len = parse_long(fp);

    if ( getc(fp) != 'H' )
    {
	kom_errno = KOM_SERVER_IS_CRAZY;
	return FAILURE;
    }

    result->string = REALLOC(result->string, result->len);

    if ( result->string == NULL )
    {
	kom_errno = KOM_OUT_OF_MEMORY;
	return FAILURE;
    }
    
    for ( i = 0; i < result->len; i++ )
    {
	if ( (c=getc(fp)) == EOF )
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	result->string[ i ] = c;
    }
    

    return OK;
}


extern Success
parse_member_list(FILE *fp,
		  Member_list *result)
{
    int i;

    result->no_of_members = parse_long(fp);
    if ( result->no_of_members > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->members = REALLOC(result->members,
				      (result->no_of_members
				       * sizeof(Member)));

	    if ( result->members == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_members; i++)
		if ( parse_member(fp, &result->members[ i ]) != OK )
		{
		    return FAILURE;
		}
	    
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->members != NULL )
	    {
		isc_free(result->members);
		result->members = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->members != NULL )
	{
	    isc_free(result->members);
	    result->members = NULL;
	}
    }
    return OK;
}


extern Success
parse_member(FILE *fp,
	     Member *result)
{
    result->member = parse_long(fp);
    return OK;
}

extern Success
parse_mark(FILE *fp,
	   Mark *result)
{
    result->text_no = parse_long(fp);
    result->mark_type = parse_long(fp);
    return OK;
}

extern Success
parse_misc_info(FILE *fp, 
		Misc_info *result)
{
    result->type = parse_long(fp);
    
    switch(result->type)
    {
    case recpt:
	result->datum.recipient = parse_long(fp);
	break;
	
    case cc_recpt:
	result->datum.cc_recipient = parse_long(fp);
	break;
	
    case loc_no:
	result->datum.local_no = parse_long(fp);
	break;
	
    case rec_time:
	parse_time(fp, &result->datum.received_at);
	break;
	
    case comm_to:
	result->datum.comment_to = parse_long(fp);
	break;
	
    case comm_in:
	result->datum.commented_in = parse_long(fp);
	break;
	
    case footn_to:
	result->datum.footnote_to = parse_long(fp);
	break;
	
    case footn_in:
	result->datum.footnoted_in = parse_long(fp);
	break;
	
    case sent_by:
	result->datum.sender = parse_long(fp);
	break;
	
    case sent_at:
	parse_time(fp, &result->datum.sent_at);
	break;

    default:
	fprintf(stderr, "parse_misc_info: Illegal misc\n");
	fflush(stdout);
	fflush(stderr);
	abort();
    }
    return OK;
}

extern Success
parse_priv_bits(FILE *fp,
		Priv_bits *result)
{
    skipwhite(fp);

    result->wheel = getc(fp) != '0';
    result->admin = getc(fp) != '0';
    result->statistic = getc(fp) != '0';
    result->create_pers = getc(fp) != '0';
    result->create_conf = getc(fp) != '0';
    result->change_name = getc(fp) != '0';
    result->extern_gw = getc(fp) != '0';
    result->flg8 = getc(fp) != '0';
    result->flg9 = getc(fp) != '0';
    result->flg10 = getc(fp) != '0';
    result->flg11 = getc(fp) != '0';
    result->flg12 = getc(fp) != '0';
    result->flg13 = getc(fp) != '0';
    result->flg14 = getc(fp) != '0';
    result->flg15 = getc(fp) != '0';
    result->flg16 = getc(fp) != '0';

    return OK;
}


extern Success
parse_personal_flags(FILE *fp,
		     Personal_flags *result)
{
    skipwhite(fp);
    
    result->unread_is_secret = getc(fp) != '0';
    result->flg2 = getc(fp) != '0';
    result->flg3 = getc(fp) != '0';
    result->flg4 = getc(fp) != '0';
    result->flg5 = getc(fp) != '0';
    result->flg6 = getc(fp) != '0';
    result->flg7 = getc(fp) != '0';
    result->flg8 = getc(fp) != '0';

    return OK;
}	

extern Success
parse_conf_type(FILE *fp,
		Conf_type *result)
{
    skipwhite(fp);
    
    result->rd_prot = getc(fp) != '0';
    result->original = getc(fp) != '0';
    result->secret = getc(fp) != '0';
    result->letter_box = getc(fp) != '0';

    return OK;
}


extern Success
parse_who_info(FILE *fp,
	       Who_info *result)
{
    result->person = parse_long(fp);
    result->working_conference = parse_long(fp);
    result->session_no = parse_long(fp);
    
    if ( parse_string(fp, &result->what_am_i_doing) != OK
	|| parse_string(fp, &result->username) != OK )
    {
	kom_errno = KOM_SERVER_IS_CRAZY;
	return FAILURE;
    }

    return OK;
}

    
extern Success
parse_who_info_list(FILE *fp,
		     Who_info_list *result)
{
    int i;
    
    skipwhite(fp);

    result->no_of_persons = parse_long(fp);
    
    if ( result->no_of_persons > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->info = REALLOC(result->info,
				   (result->no_of_persons
				    * sizeof(Who_info)));

	    if ( result->info == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_persons; i++)
	    {
		result->info[ i ] = EMPTY_WHO_INFO;
		if ( parse_who_info(fp, &result->info[ i ]) != OK )
		    return FAILURE;
	    }
	    
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->info != NULL )
	    {
		isc_free(result->info);
		result->info = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->info != NULL )
	{
	    isc_free(result->info);
	    result->info = NULL;
	}
    }
    return OK;
}
    
extern Success
parse_conf_no_list(FILE *fp,
		   Conf_no_list *result)
{
    int i;
    
    skipwhite(fp);

    result->no_of_confs = parse_long(fp);
    
    if ( result->no_of_confs > 0 )
    {
	skipwhite(fp);
	switch(getc(fp))
	{
	case '{':
	    result->conf_nos = REALLOC(result->conf_nos,
				       (result->no_of_confs
					* sizeof(Conf_no)));

	    if ( result->conf_nos == NULL )
	    {
		kom_errno = KOM_OUT_OF_MEMORY;
		return FAILURE;
	    }
	    
	    for ( i = 0; i < result->no_of_confs; i++)
	    {
		result->conf_nos[ i ] = parse_long(fp);
	    }
	    
	    skipwhite(fp);
	    if ( getc(fp) != '}' )
	    {
		kom_errno = KOM_SERVER_IS_CRAZY;
		return FAILURE;
	    }
	    
	    break;

	case '*':
	    if ( result->conf_nos != NULL )
	    {
		isc_free(result->conf_nos);
		result->conf_nos = NULL;
	    }
	    break;

	default:
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
    }
    else
    {
	skipwhite(fp);
	if ( getc(fp) != '*' ) 
	{
	    kom_errno = KOM_SERVER_IS_CRAZY;
	    return FAILURE;
	}
	if ( result->conf_nos != NULL )
	{
	    isc_free(result->conf_nos);
	    result->conf_nos = NULL;
	}
    }
    return OK;
}

extern Success
parse_time(FILE *fp, struct tm *timeptr)
{
    timeptr->tm_sec = parse_long(fp);
    timeptr->tm_min = parse_long(fp);
    timeptr->tm_hour = parse_long(fp);
    timeptr->tm_mday = parse_long(fp);
    timeptr->tm_mon = parse_long(fp);
    timeptr->tm_year = parse_long(fp);
    timeptr->tm_wday = parse_long(fp);
    timeptr->tm_yday = parse_long(fp);
    timeptr->tm_isdst = parse_long(fp);

    return OK;
}
