/*
 * $Id: output.c,v 0.7 1992/04/04 17:21:45 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 *  output.c  --  initiate call to client.
 *
 *  Created by ceder 1990-07-17
 */

static char *rcsid = "$Id: output.c,v 0.7 1992/04/04 17:21:45 ceder Exp $";

#include <stdio.h>

#include <kom-types.h>
#include <config.h>
#include <services.h>

#include "isc-client.h"
#include "isc-output.h"
#include "services.h"
#include <com.h>

#include "output.h"

static void
output_packet_header(Call_header fnc)
{
    rewind(KS->fp);
    fprintf(KS->fp, "0 %lu", (u_long)fnc);
}


extern  void
output_login_old (Pers_no	person,
		  const String	passwd)
{
    output_packet_header(call_fnc_login_old);
    fprintf(KS->fp, " %lu", (u_long)person);
    output_string(KS->fp, passwd);
    putc('\n', KS->fp);
    fflush(KS->fp);
}

extern  void
output_login (Pers_no		person,
	      const String	passwd,
	      Bool		visible)
{
    output_packet_header(call_fnc_login);
    fprintf(KS->fp, " %lu", (u_long)person);
    output_string(KS->fp, passwd);
    fprintf(KS->fp, " %lu", (u_long)visible);
    putc('\n', KS->fp);
    fflush(KS->fp);
}



extern  void
output_logout (void)	/* can never fail */
{
    output_packet_header(call_fnc_logout);
    putc('\n', KS->fp);
    fflush(KS->fp);
}



extern  void
output_pepsi (Conf_no	conference )
{
    output_packet_header(call_fnc_pepsi);
    fprintf(KS->fp, " %lu\n", (u_long)conference);
    fflush(KS->fp);
}




extern  void
output_change_name (Conf_no       conf_no,
		    const String  new_name)
{
    output_packet_header(call_fnc_change_name);
    fprintf(KS->fp, " %lu", (u_long)conf_no);
    output_string(KS->fp, new_name );
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern  void
output_change_what_i_am_doing (const String  what_am_i_doing )
{
    output_packet_header(call_fnc_change_what_i_am_doing);
    output_string(KS->fp, what_am_i_doing );
    putc('\n', KS->fp);
    fflush(KS->fp);
}




/******************************
*     Person-related calls    *
******************************/

/*
 *  Create a new person.  Returns 0 if any error occured.
 */
extern void
output_create_person (const String  name,
		      const String  passwd )
{
    output_packet_header(call_fnc_create_person);
    output_string(KS->fp, name );
    output_string( KS->fp, passwd );
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern  void
output_get_person_stat (Pers_no	        person)
{
    output_packet_header(call_fnc_get_person_stat);
    fprintf(KS->fp, " %lu\n", (u_long) person);
    fflush(KS->fp);
}

extern  void
output_get_created_texts(Pers_no       pers_no,
			 Local_text_no first,
			 u_long        len )
{
    output_packet_header(call_fnc_get_created_texts);
    fprintf(KS->fp, " %lu %lu %lu\n",
	    (u_long) pers_no,
	    (u_long) first,
	    (u_long) len);
    fflush(KS->fp);
}


extern  void
output_get_membership (Pers_no		  person,
		       u_short		  first,
		       u_short		  no_of_confs,
		       Bool		  want_read_texts )
{
    output_packet_header(call_fnc_get_membership);
    fprintf(KS->fp, " %lu %lu %lu %lu\n",
	    (u_long)person,
	    (u_long)first,
	    (u_long)no_of_confs,
	    (u_long)want_read_texts);
    fflush(KS->fp);
}


extern  void
output_set_priv_bits (Pers_no	 person,
		      Priv_bits  privileges )
{
    output_packet_header(call_fnc_set_priv_bits);
    fprintf(KS->fp, " %lu", (u_long) person);
    output_priv_bits( KS->fp, privileges );
    putc('\n', KS->fp);
    fflush(KS->fp);
}



extern  void
output_set_passwd (Pers_no	  person,
		   const String   old_pwd,
		   const String   new_pwd )
{
    output_packet_header(call_fnc_set_passwd);
    fprintf(KS->fp, " %lu", (u_long) person);
    output_string( KS->fp, old_pwd );
    output_string( KS->fp, new_pwd );
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern  void
output_query_read_texts (Pers_no	   pers_no,
			 Conf_no	   conf_no)
{
    output_packet_header(call_fnc_query_read_texts);
    fprintf(KS->fp, " %lu %lu\n", (u_long) pers_no, (u_long) conf_no);
    fflush(KS->fp);
}




/****************************************
*	Conference-related calls	*
****************************************/


extern  void
output_create_conf (const String  name,
		    Conf_type	  type )
{
    output_packet_header(call_fnc_create_conf);
    output_string( KS->fp, name );
    output_conf_type( KS->fp, type );
    putc('\n', KS->fp);
    fflush(KS->fp);
}



extern  void
output_delete_conf (Conf_no	conf )
{
    output_packet_header(call_fnc_delete_conf);
    fprintf(KS->fp, " %lu\n", (u_long) conf);
    fflush(KS->fp);
}



extern  void
output_lookup_name (const String   name)
{
    output_packet_header(call_fnc_lookup_name);
    output_string( KS->fp, name );
    putc('\n', KS->fp);
    fflush(KS->fp);
}




extern  void
output_get_conf_stat (Conf_no  conf_no)
{
    output_packet_header(call_fnc_get_conf_stat);
    fprintf(KS->fp, " %lu\n", (u_long) conf_no);
    fflush(KS->fp);
}

extern  void
output_get_members (Conf_no	  conf,
		    u_short	  first,
		    u_short	  no_of_members)
{
    output_packet_header(call_fnc_get_members);
    fprintf(KS->fp, " %lu %lu %lu\n", (u_long) conf, (u_long) first,
	    (u_long) no_of_members);
    fflush(KS->fp);
}


extern  void
output_add_member (Conf_no	conf_no,
		   Pers_no	pers_no,
		   u_char	priority,
		   u_short	where	)
{
    output_packet_header(call_fnc_add_member);
    fprintf(KS->fp, " %lu %lu %lu %lu\n", (u_long) conf_no, (u_long) pers_no,
	    (u_long) priority, (u_long) where);
    fflush(KS->fp);
}



extern  void
output_sub_member (Conf_no	conf_no,
		   Pers_no	pers_no )
{
    output_packet_header(call_fnc_sub_member);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) pers_no);
    fflush(KS->fp);
}


extern  void
output_set_unread (Conf_no   conf_no,
		   Text_no   no_of_unread)
{
    output_packet_header(call_fnc_set_unread);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) no_of_unread);
    fflush(KS->fp);
}



extern  void
output_set_presentation (Conf_no   conf_no,
			 Text_no   text_no )
{
    output_packet_header(call_fnc_set_presentation);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) text_no);
    fflush(KS->fp);
}


extern  void
output_set_etc_motd (Conf_no	  conf_no,
		     Text_no	  text_no )
{
    output_packet_header(call_fnc_set_etc_motd);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) text_no);
    fflush(KS->fp);
}



extern  void
output_set_supervisor (Conf_no	  conf_no,
		       Conf_no	  admin	)
{
    output_packet_header(call_fnc_set_supervisor);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) admin);
    fflush(KS->fp);
}


extern  void
output_set_permitted_submitters (Conf_no  conf_no,
				 Conf_no  perm_sub )
{
    output_packet_header(call_fnc_set_permitted_submitters);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) perm_sub);
    fflush(KS->fp);
}


extern  void
output_set_super_conf (Conf_no	  conf_no,
		       Conf_no	  super_conf )
{
    output_packet_header(call_fnc_set_super_conf);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) super_conf);
    fflush(KS->fp);
}


extern  void
output_set_conf_type (Conf_no	 conf_no,
		      Conf_type  type	)
{
    output_packet_header(call_fnc_set_conf_type);
    fprintf(KS->fp, " %lu", (u_long) conf_no);
    output_conf_type(KS->fp, type);
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern  void
output_set_garb_nice (Conf_no	 conf_no,
		      Garb_nice  nice	)
{
    output_packet_header(call_fnc_set_garb_nice);
    fprintf(KS->fp, " %lu %lu\n", (u_long) conf_no, (u_long) nice);
    fflush(KS->fp);
}




/********************************
*     Calls to handle marks     *
********************************/


extern  void
output_get_marks (void)
{
    output_packet_header(call_fnc_get_marks);
    putc('\n', KS->fp);
    fflush(KS->fp);
}




extern  void
output_mark_text (Text_no  text,
		  u_char   mark_type )
{
    output_packet_header(call_fnc_mark_text);
    fprintf(KS->fp, " %lu %lu\n", (u_long) text, (u_long) mark_type);
    fflush(KS->fp);
}





/*******************************
*     Calls to handle texts    *
*******************************/


extern  void
output_get_text (Text_no      text,
		 String_size  start_char,
		 String_size  end_char)
{
    output_packet_header(call_fnc_get_text);
    fprintf(KS->fp, " %lu %lu %lu\n", (u_long)text,
	    (u_long)start_char, (u_long)end_char);
    fflush(KS->fp);
}



extern  void
output_get_text_stat (Text_no    text)
{
    output_packet_header(call_fnc_get_text_stat);
    fprintf(KS->fp, " %lu\n", (u_long)text);
    fflush(KS->fp);
}


extern  void
output_mark_as_read (Conf_no		    conference,
		     int		    no_of_texts,
		     const Local_text_no  * text_arr   )
{
    int i;
    
    output_packet_header(call_fnc_mark_as_read);
    fprintf(KS->fp, " %lu %lu", (u_long) conference, (u_long) no_of_texts);
    fprintf(KS->fp, " {");
    for ( i = 0; i < no_of_texts; i++ )
	fprintf(KS->fp, " %lu", (u_long)text_arr[ i ]);
    fprintf(KS->fp, " }\n");
    fflush(KS->fp);
}



extern  void
output_create_text (const String       message,
		    u_short	       no_of_misc,
		    const Misc_info  * misc	     )
{
    int i;
    
    output_packet_header(call_fnc_create_text);
    output_string( KS->fp, message );
    fprintf(KS->fp, " %lu {", (u_long)no_of_misc);
    for ( i = 0; i < no_of_misc; i++ )
	output_misc_info(KS->fp, misc[ i ]);
    fprintf(KS->fp, " }\n");
    fflush(KS->fp);
}

extern  void
output_create_anonymous_text (const String       message,
			      u_short	       no_of_misc,
			      const Misc_info  * misc	     )
{
    int i;
    
    output_packet_header(call_fnc_create_text);
    output_string( KS->fp, message );
    fprintf(KS->fp, " %lu {", (u_long)no_of_misc);
    for ( i = 0; i < no_of_misc; i++ )
	output_misc_info(KS->fp, misc[ i ]);
    fprintf(KS->fp, " }\n");
    fflush(KS->fp);
}


extern void
output_delete_text( Text_no text_no )
{
    output_packet_header(call_fnc_delete_text);
    fprintf(KS->fp, " %lu\n", (u_long) text_no);
    fflush(KS->fp);
}




extern  void
output_add_recipient (Text_no	  text_no,
		      Conf_no	  conf_no,
		      Info_type  type	)
{
    output_packet_header(call_fnc_add_recipient);
    fprintf(KS->fp, " %lu %lu %lu\n", (u_long) text_no,
	    (u_long) conf_no, (u_long) type);
    fflush(KS->fp);
}


extern  void
output_sub_recipient (Text_no	  text_no,
		      Conf_no	  conf_no )
{
    output_packet_header(call_fnc_sub_recipient);
    fprintf(KS->fp, " %lu %lu\n", (u_long) text_no, (u_long) conf_no);
    fflush(KS->fp);
}



extern  void
output_add_comment (Text_no	  comment,
		    Text_no 	  comment_to )
{
    output_packet_header(call_fnc_add_comment);
    fprintf(KS->fp, " %lu %lu\n", (u_long) comment, (u_long) comment_to);
    fflush(KS->fp);
}



extern  void
output_sub_comment (Text_no	  comment,
		    Text_no	  comment_to )
{
    output_packet_header(call_fnc_sub_comment);
    fprintf(KS->fp, " %lu %lu\n", (u_long) comment, (u_long) comment_to);
    fflush(KS->fp);
}

extern  void
output_add_footnote (Text_no	  footnote,
		     Text_no 	  footnote_to )
{
    output_packet_header(call_fnc_add_footnote);
    fprintf(KS->fp, " %lu %lu\n", (u_long) footnote, (u_long) footnote_to);
    fflush(KS->fp);
}



extern  void
output_sub_footnote (Text_no	  footnote,
		     Text_no	  footnote_to )
{
    output_packet_header(call_fnc_sub_footnote);
    fprintf(KS->fp, " %lu %lu\n", (u_long) footnote, (u_long) footnote_to);
    fflush(KS->fp);
}



extern  void
output_get_map (Conf_no	       conf_no,
		Local_text_no  first_local_no,
		Local_text_no  no_of_texts)
{
    output_packet_header(call_fnc_get_map);
    fprintf(KS->fp, " %lu %lu %lu\n", (u_long) conf_no,
	    (u_long) first_local_no, (u_long) no_of_texts);
    fflush(KS->fp);
}



extern  void
output_get_time ( void )
{
    output_packet_header(call_fnc_get_time);
    putc('\n', KS->fp);
    fflush(KS->fp);
}

extern  void
output_who_is_on ( void )
{
    output_packet_header(call_fnc_who_is_on);
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern  void
output_get_info ( void )
{
    output_packet_header(call_fnc_get_info);
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern void
output_set_motd_of_lyskom (Text_no motd)
{
    output_packet_header(call_fnc_set_motd_of_lyskom);
    fprintf(KS->fp, " %lu\n", (u_long) motd);
    fflush(KS->fp);
}


extern void
output_enable (u_char ena_level)
{
    output_packet_header(call_fnc_enable);
    fprintf(KS->fp, " %lu\n", (u_long) ena_level);
    fflush(KS->fp);
}


extern void
output_sync (void) 
{
    output_packet_header(call_fnc_sync);
    putc('\n', KS->fp);
    fflush(KS->fp);
}


extern void
output_shutdown (int exit_val) 
{
    output_packet_header(call_fnc_shutdown);
    fprintf(KS->fp, " %lu\n", (u_long) exit_val);
    fflush(KS->fp);
}


extern void
output_broadcast (const String message) 
{
    output_packet_header(call_fnc_broadcast);
    output_string(KS->fp, message);
    putc('\n', KS->fp);
    fflush(KS->fp);
}

extern void
output_send_message (Pers_no recipient,
		     const String message) 
{
    output_packet_header(call_fnc_send_message);
    fprintf(KS->fp, " %lu\n", (u_long) recipient);
    output_string(KS->fp, message);
    putc('\n', KS->fp);
    fflush(KS->fp);
}

void
output_get_unread_confs (Pers_no pers_no)
{
    output_packet_header(call_fnc_get_unread_confs);
    fprintf(KS->fp, " %lu\n", (u_long) pers_no);
    fflush(KS->fp);
}

