/*
 *  client/main.c  --  LysKOM client, main program
 *
 *
 *  Copyright (C) 1990  Lysator Computer Club
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  E-mail:	Bellman@Lysator.LiU.SE
 */


#include <stdio.h>
#include <stdlib.h>

#include <zmalloc.h>
#include <s-string.h>

#include "config.h"
#include <kom-types.h>
#include <services.h>

#include "version.h"

#include "error.h"
#include "read-line.h"
#include "texts.h"
#include "commands.h"
#include "cmds.h"
#include "cache.h"
#include "xoutput.h"

#include "defaults.h"

Info		  kom_info;	/*  LysKOM server parameters  */

Old_tty		  save_tty_param;


int
main (int	   argc,
      char	** argv)

{
#define FUNCTION "main()"

    Info	serv_info;
    int         server_arg;
    s_set_storage_management (zmalloc, zrealloc, zfree);
    kom_set_storage_management (zmalloc, zrealloc, zfree);
    init_messages();	/* Initialize the 'txts' struct */
    init_cmd_list();	/* Initialize the list of commands */
    init_terminal (stdin, stdout, &save_tty_param);
    init_cache();

    /* Aronsson was here: 7 bit character size conversion for terminals */
    /* For now, argument parsing is very primitive. */
    /* Perhaps we should consider GNU's getopt. */
    if (argc > 1 && 0 == strcmp (argv[1], "--swascii"))
      {
	xo_cs7 = 1;
	server_arg = 2;
      }
    else
      server_arg = 1;

    if (kom_open_connection(argv[server_arg]
			    ? argv[server_arg]
			    : DEFAULT_SERVER) == FAILURE)
    {
	fatal2 (CLIENT_SERVER_ERROR,
		"Can't connect to server error:", kom_errno);
	exit(1);
    }

    /*  OK, connection established  */
    kom_get_info (&serv_info);	/* BUG! Retval not checked */

    /*  Greet user  */
    xprintf (txts.greet_user /* , serv_info.server_id  Vad {r detta? */,
	     serv_info.version,
	     VERSION_ID);

    /*  Log in to server  */
    cmd_start_anew (EMPTY_STRING);
    
    /*  And run the command loop  */
    command_loop ();

    /* Restore the terminal to the old settings */
    restore_terminal (stdin, stdout, &save_tty_param);

    return OK;	/*  LysKOM can *never* be wrong! */
#undef FUNCTION
}

