/*
 *  xoutput.c  --  Print text to a file, and if it is a tty, then stop
 *		   and wait when printed a screenful.
 *	
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/termios.h>
#include <sgtty.h>
#include <errno.h>
#include <signal.h>

#include <s-string.h>

#include "error.h"
#include "edit.h"
#include "texts.h"
#include "quit.h"

#include "do-printf.h"
#include "xoutput.h"


#define	Export


static  XO_stream	  kom_dfl_stream = { XO_terminal, NULL, -1, 0 };


/*  ==================================================================  */
/*		    	    Global variables				*/

Export  XO_stream	* kom_outstream		= &kom_dfl_stream;
Export	int		  printed_lines;
Export	int		  current_column;

/* Default character size is 8 bits */
Export  int               xo_cs7 		= 0;


/*  ==================================================================  */
/*			    Local variables				*/

static	int		  window_width	= 80;	/* Default value */
static	int		  window_height	= 24;	/* Default value */




/*  ==================================================================  */
/*		    Forward declarations of functions			*/

static  int	do_xputc (int ch, void *stream);

static  int	printf_String  (va_list    * arg_p,
				putc_fun_T   putc_func,
				char	     sign,
				int	     flags,
				int	     width,
				int	     precision,
				void	   * stream);

static  int	printf_goto_column (va_list	 * arg_p,
				    putc_fun_T	   putc_func,
				    char	   sign,
				    int		   flags,
				    int		   width,
				    int		   precision,
				    void	 * stream);




/*  ==================================================================  */
/*			Constants and other things			*/

#define	BUFFER_SIZE	  8192		/* Max length of one printout */



void
catch_sigwinch() {
#define FUNCTION "catch_sigwinch()"
    struct winsize	  sizes;
    
    if (ioctl( 0, TIOCGWINSZ, &sizes)) {
	perror("Oops! Problem with ioctl():");
	fatal2 (CLIENT_ERROR_TOO_boring,
		"Setting tty parameters. Error: %d", errno);
    }
    if (sizes.ws_row)
	window_height = sizes.ws_row;
    if (sizes.ws_col)
	window_width = sizes.ws_col;
    signal(SIGWINCH, catch_sigwinch);
#undef FUNCTION
}

/*  ==================================================================  */
/*			    Exported functions				*/


/*
 *  Do some initializations for the terminal and XO-output functions.
 *  Save the old terminal settings in *OLD_TTY_PARAM.
 */
Export  Success
init_terminal (FILE	* infile,
	       FILE	* outfile,
	       Old_tty	* old_tty_param)

{
#define FUNCTION "init_terminal()"
    struct  sgttyb	  tty_param;
    int			  infileno;
    struct winsize	  sizes;
    
    extern void		  suspend_lyskom();

    extern int		  errno;

    infileno = fileno(infile);
    /* Fetch tty settings */
    if (ioctl (infileno, TIOCGETP, (caddr_t) old_tty_param) == -1)
    {
    	perror("Oops!  You shouldn't see this.  Anyway, here's the error:\n"
	       "Fetching tty parameters: ");
	fatal2 (CLIENT_ERROR_TOO_boring, 
	       "Fetching tty parameters. Error: %d", errno);
    }

    /*  Turn off echo and on cbreak  */
    tty_param = *old_tty_param;
    tty_param.sg_flags = (tty_param.sg_flags | CBREAK) & (~ ECHO);
    if (ioctl(infileno, TIOCSETP, (caddr_t) &tty_param))
    {
    	perror("Oops!  You shouldn't see this.  Anyway, here's the error:\n"
	       "Setting tty parameters: ");
	fatal2 (CLIENT_ERROR_TOO_boring,
	       "Setting tty parameters. Error: %d", errno);
    }

    if (ioctl(infileno, TIOCGWINSZ, &sizes)) {
	perror("Oops! Problem with ioctl():");
	fatal2 (CLIENT_ERROR_TOO_boring,
	       "Setting tty parameters. Error: %d", errno);
    }
    if (sizes.ws_row)
	window_height = sizes.ws_row;
    else if (getenv("LINES")
	     && atoi(getenv("LINES")) > 0)
	window_height = atoi(getenv("LINES"));
    if (sizes.ws_col)
	window_width = sizes.ws_col;
    else if (getenv("COLUMNS")
	     && atoi(getenv("COLUMNS")) > 0)
	window_width = sizes.ws_col;

    extend_printf ('S', &printf_String);
    extend_printf ('T', &printf_goto_column);

    signal(SIGTSTP, suspend_lyskom);
    signal(SIGWINCH, catch_sigwinch);

    return  OK;		/* Well, in the first version... */
}


/*
 *  Restore the terminal settings as saved in *OLD_TTY_PARAM.
 */
Export  Success
restore_terminal (FILE		* infile,
		  FILE		* outfile,
		  Old_tty	* old_tty_param)

{
    ioctl(fileno(infile), TIOCSETP, (caddr_t) old_tty_param);
    return  OK;
}


/* What to do when suspending */
String *theprompt;
void
suspend_lyskom() {
    struct sgttyb	saved;

    restore_terminal(stdin, stdout, &saved);
    kill (getpid(), SIGSTOP);
    init_terminal(stdin, stdout, &saved);
    if (theprompt) {
	xprintf("\n%S - ", *theprompt);
	fflush(stdout);
    }
}


/* Print a single character to a XO_stream. */
Export  int
xputc (char	      char_to_print,
       XO_stream    * outstream)
{
    return  do_xputc (char_to_print, (void *)outstream);
}   /* END: xputchar() */



Export  int
s_xfputs (String	  str,
	  XO_stream	* outstream)

{
    String_size		  pos;


    for ( pos = 0 ;  pos < s_strlen (str) ;  pos++ )
	do_xputc (str.string[pos], (void *)outstream);

    return  s_strlen (str);
}   /* END: s_xsputs() */



/*
 *  Print the argument list ARGS to OUTSTREAM according to the format string
 *  FORMAT.
 */
Export  int
vxfprintf (XO_stream	* outstream,
	   const char	* format,
	   va_list	  args	    )

{
    return  do_printf (&do_xputc, format, args, (void *)outstream);
}   /* END: vxfprintf */





Export  int
xprintf (const char	* format,
	 ...			 )

{
    va_list		  args;
    int			  retval;

    va_start (args, format);
    retval = do_printf (&do_xputc, format, args, kom_outstream);
    va_end (args);
    return  retval;
}   /* END: xprintf() */



/*
 *  Flush all pending output to the XO_stream OUTSTREAM.
 */
Export  int
xflush (XO_stream	* outstream)
{
    String		  temp_str;

    switch (outstream->stream_type)
    {
    case  XO_terminal:
	fflush (stdout);
	break;

    case  XO_file:
	fflush (outstream->f_stream);
	break;

    case  XO_edited_text:
	temp_str.len = outstream->used_buf;
	temp_str.string = outstream->buffer;
	edit_add_string (outstream->text_no, temp_str);
	outstream->used_buf = 0;
	break;
    }

    return  0;
}



/*
 *  Move cursor to a specific column.  Moves to that column on the next
 *  line if cursor has passed that column already.  Wraps if specified
 *  column is outside screen.
 */
Export  int
xo_goto_column (int		  column,
		XO_stream	* outstream)

{
    return  printf_goto_column (NULL, &do_xputc, '\0', 0,
				column, 0, (void *)outstream);
}



/*  ==================================================================  */
/*			    Local functions				*/

static int				/* ARONSSON WAS HERE: */
  xo_shrink (int ch)			/* shrink a char to 7 bits */
{
  if (ch < 0200)
    return ch;
  switch (ch)
    {
    default: return 0200 & ch;		/* a bit primitive for now */
    case 0304: return '[';      case 0344: return '{';
    case 0305: return ']';      case 0345: return '}';
    case 0311: return 'E';      case 0351: return 'e';
    case 0315: return 'I';      case 0355: return 'i';
    case 0326: return '\\';     case 0366: return '|';
    }
}

/*
 *  Print one character to a XO_stream, as called from do_printf().
 */
static  int
do_xputc (int	  ch,
	  void	* stream)
{
    int		  to_next_tab;
    XO_stream	* outstream	= (XO_stream *)stream;

    if (ch < 0)				/* Garlic against signed chars */
      ch += 0400;
    
    switch (ch)
    {
    case  '\n':
	printed_lines++;
	current_column = 0;
	break;

    case  '\r':
	current_column = 0;
	break;

    case  '\b':
	current_column--;
	break;

    case  '\t':
	/* Assumes tab stops every 8th position */
	to_next_tab = 8 - current_column % 8;
	current_column += to_next_tab;
	break;

    case  '\a':
	break;

    default:
	current_column++;
	break;
    }  /* switch (ch) */


    switch (outstream->stream_type)
    {
    case  XO_file:
        exit (1);
	/* ARONSSON WAS HERE: 26 JAN 1993
	 * This is a foul bug, which has never been run.
	 * xputc calls the present function,
	 * so the present function should not call xputc.
	 * instead, putc should be called with the
	 * appropriate FILE* stream.
	 */
	xputc (ch, outstream);
	break;

    case  XO_edited_text:
	outstream->buffer[outstream->used_buf++] = ch;

	if (outstream->used_buf >= XO_EDIT_BUFSIZE)
	{
	    xflush (outstream);
	    outstream->used_buf -= XO_EDIT_BUFSIZE;
	}
	break;
	

    case  XO_terminal:
	if (xo_cs7 && ch > 127)
	  putc (xo_shrink (ch), stdout); /* Aronsson was here */
	else
	  putc (ch, stdout);
	while (current_column >= window_width)/* 'if' should suffice... */
	{
	    /* Hmm... I'll assume the window is at least 8 characters
	     * wide.  This is for tab characters....		*/
	    current_column -= window_width;
	    printed_lines++;
	}

	/* Check if we have printed a screenful */
	if (printed_lines >= window_height - 1)	/* MAGIC NUMBER! */
	{
	    int		  c, i;
	    FILE	* instream;

	    /* This is the only sensible file I know of. */
	    instream = stdin;
	    /* Cannot call xputs here... */
	    /* xputs (txts.Press_return); */
	    c = strlen (txts.Press_return);
	    if (xo_cs7)
	      for (i = 0; i < c; i++)
		putchar (xo_shrink ((unsigned char)
				    txts.Press_return[i]));
	    else
	        fputs (txts.Press_return, stdout);
	    fflush (stdout);
	    /* Assuming cbreak and no echo */
	    do
	    {
		c = getc (instream);
	    } while (    c != '\r'  &&  c != '\n'
		     &&  c != ' '   &&  c != 'q'  &&  c != 'Q');
	    putchar ('\r');
	    for (i = strlen (txts.Press_return) ;  i > 0 ;  i--)
		putchar (' ');
	    putchar ('\r');
	    printed_lines = 0;
	    current_column = 0;
	    if (c == 'q'  ||  c == 'Q')
		longjmp (toploop_jmpbuf, 17);
	}


	break;

    }  /* switch (kom_outstream->stream_type) */

    return  1;
}   /* END: do_xputc() */



/*
 *  Handle the 'S' format for do_printf(), i e an argument of type 'String'.
 */
static  int
printf_String  (va_list	   * arg_p,
		putc_fun_T   putc_func,
		char	     sign,
		int	     flags,
		int	     width,
		int	     precision,
		void	   * stream)

{
    String		   str;
    String_size		   i;


    str = va_arg (*arg_p, String);

    /* BUG!  Doesn't handle sign, flags, width och precision. */
    for ( i = 0 ;  i < s_strlen (str) ;  i++ )
	(*putc_func)(str.string[i], stream);

    return  s_strlen (str);
}   /* END: printf_String() */


#define BUF_SZ		128

static  int
printf_goto_column (va_list	 * arg_p,
		    putc_fun_T	   putc_func,
		    char	   sign,
		    int		   flags,
		    int		   column,
		    int		   precision,
		    void	 * stream)

{
    int		  no_of_tabs;
    int		  no_of_spaces;
    int		  no_of_printed_chars;

    if (current_column >= column)
	(*putc_func) ('\n', stream);

    no_of_tabs = column/8 - current_column/8;
    no_of_spaces = column - (current_column/8 + no_of_tabs)*8;
    no_of_printed_chars = no_of_tabs + no_of_spaces;

    while (no_of_tabs-- > 0)
	(*putc_func) ('\t', stream);
    while (no_of_spaces-- > 0)
	(*putc_func) (' ', stream);

    return  no_of_printed_chars;
}   /* END: printf_goto_column() */
